package proxy.packet.play.client;

import java.io.IOException;

import proxy.network.NetHandlerPlayServer;
import proxy.network.Packet;
import proxy.network.PacketBuffer;
import proxy.util.Vec3;
import proxy.util.ChatComponent;

public class C12PacketUpdateSign implements Packet<NetHandlerPlayServer>
{
    private Vec3 pos;
    private ChatComponent[] lines;

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.pos = buf.readVector();
        this.lines = new ChatComponent[4];

        for (int i = 0; i < 4; ++i)
        {
            String s = buf.readStringFromBuffer(384);
            ChatComponent ichatcomponent = ChatComponent.Serializer.jsonToComponent(s);
            this.lines[i] = ichatcomponent;
        }
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVector(this.pos);

        for (int i = 0; i < 4; ++i)
        {
            ChatComponent ichatcomponent = this.lines[i];
            String s = ChatComponent.Serializer.componentToJson(ichatcomponent);
            buf.writeString(s);
        }
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(NetHandlerPlayServer handler)
    {
        handler.processUpdateSign(this);
    }

    public ChatComponent[] getLines()
    {
        return this.lines;
    }
}
