package proxy.packet.status.server;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.io.IOException;

import proxy.network.NetHandlerStatusClient;
import proxy.network.Packet;
import proxy.network.PacketBuffer;
import proxy.util.ChatComponent;
import proxy.util.ChatStyle;
import proxy.util.JsonEnumFactory;
import proxy.util.PlayerInfo;
import proxy.util.ServerInfo;
import proxy.util.VersionId;

public class S00PacketServerInfo implements Packet<NetHandlerStatusClient>
{
    private static final Gson GSON = (new GsonBuilder()).registerTypeAdapter(VersionId.class, new VersionId.Serializer()).registerTypeAdapter(PlayerInfo.class, new PlayerInfo.Serializer()).registerTypeAdapter(ServerInfo.class, new ServerInfo.Serializer()).registerTypeHierarchyAdapter(ChatComponent.class, new ChatComponent.Serializer()).registerTypeHierarchyAdapter(ChatStyle.class, new ChatStyle.Serializer()).registerTypeAdapterFactory(new JsonEnumFactory()).create();
    private ServerInfo response;

    public S00PacketServerInfo()
    {
    }

    public S00PacketServerInfo(ServerInfo responseIn)
    {
        this.response = responseIn;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.response = (ServerInfo)GSON.fromJson(buf.readStringFromBuffer(32767), ServerInfo.class);
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeString(GSON.toJson((Object)this.response));
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(NetHandlerStatusClient handler)
    {
        handler.handleServerInfo(this);
    }

    public ServerInfo getResponse()
    {
        return this.response;
    }
}
