package proxy.util;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.TypeAdapter;
import com.google.gson.TypeAdapterFactory;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;
import java.io.IOException;
import java.util.Locale;
import java.util.Map;

public class JsonEnumFactory implements TypeAdapterFactory {
	public <T> TypeAdapter<T> create(Gson gson, TypeToken<T> type) {
		Class<T> clazz = (Class<T>)type.getRawType();
		if(!clazz.isEnum())
			return null;
		final Map<String, T> map = Maps.newHashMap();
		for(T t : clazz.getEnumConstants()) {
			map.put(this.formatName(t), t);
		}
		return new TypeAdapter<T>() {
			public void write(JsonWriter writer, T data) throws IOException {
				if(data == null) {
					writer.nullValue();
				}
				else {
					writer.value(JsonEnumFactory.this.formatName(data));
				}
			}

			public T read(JsonReader reader) throws IOException {
				if(reader.peek() == JsonToken.NULL) {
					reader.nextNull();
					return (T)null;
				}
				else {
					return (T)map.get(reader.nextString());
				}
			}
		};
	}

	private String formatName(Object data) {
		return data instanceof Enum ? ((Enum)data).name().toLowerCase(Locale.US) : data.toString().toLowerCase(Locale.US);
	}
}
