package proxy.util;

import java.lang.reflect.Type;
import java.util.UUID;

import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class PlayerInfo {
	private final int capacity;
	private final int online;
	private Profile[] list;

	public PlayerInfo(int capacity, int online) {
		this.capacity = capacity;
		this.online = online;
	}

	public int getCapacity() {
		return this.capacity;
	}

	public int getOnline() {
		return this.online;
	}

	public Profile[] getList() {
		return this.list;
	}

	public void setList(Profile[] list) {
		this.list = list;
	}

	public static class Serializer implements JsonDeserializer<PlayerInfo>, JsonSerializer<PlayerInfo> {
		public PlayerInfo deserialize(JsonElement elem, Type type, JsonDeserializationContext context) throws JsonParseException {
			JsonObject obj = JsonUtils.getJsonObject(elem, "players");
			PlayerInfo info = new PlayerInfo(JsonUtils.getInt(obj, "max"), JsonUtils.getInt(obj, "online"));
			if(JsonUtils.isJsonArray(obj, "sample")) {
				JsonArray arr = JsonUtils.getJsonArray(obj, "sample");
				if(arr.size() > 0) {
					Profile[] list = new Profile[arr.size()];
					for(int z = 0; z < list.length; z++) {
						JsonObject profile = JsonUtils.getJsonObject(arr.get(z), "player[" + z + "]");
						String id = JsonUtils.getString(profile, "id");
						list[z] = new Profile(UUID.fromString(id), JsonUtils.getString(profile, "name"));
					}
					info.setList(list);
				}
			}
			return info;
		}

		public JsonElement serialize(PlayerInfo info, Type type, JsonSerializationContext context) {
			JsonObject obj = new JsonObject();
			obj.addProperty("max", info.getCapacity());
			obj.addProperty("online", info.getOnline());
			if(info.getList() != null && info.getList().length > 0) {
				JsonArray arr = new JsonArray();
				for(int z = 0; z < info.getList().length; z++) {
					JsonObject profile = new JsonObject();
					UUID id = info.getList()[z].getId();
					profile.addProperty("id", id == null ? "" : id.toString());
					profile.addProperty("name", info.getList()[z].getName());
					arr.add(profile);
				}
				obj.add("sample", arr);
			}
			return obj;
		}
	}
}