package proxy.util;

import com.google.common.collect.ForwardingMultimap;
import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.Multimap;
import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;
import java.util.Map.Entry;

public class PropertyMap extends ForwardingMultimap<String, Property> {
	private final Multimap<String, Property> properties = LinkedHashMultimap.create();

	protected Multimap<String, Property> delegate() {
		return this.properties;
	}

	public static class Serializer implements JsonSerializer<PropertyMap>, JsonDeserializer<PropertyMap> {
		public PropertyMap deserialize(JsonElement elem, Type type, JsonDeserializationContext context) throws JsonParseException {
			PropertyMap properties = new PropertyMap();
			if(elem instanceof JsonObject) {
				JsonObject obj = (JsonObject)elem;
				for(Entry<String, JsonElement> entry : obj.entrySet()) {
					if(entry.getValue() instanceof JsonArray) {
						for(JsonElement element : (JsonArray)entry.getValue()) {
							properties.put(entry.getKey(), new Property(entry.getKey(), element.getAsString()));
						}
					}
				}
			}
			else if(elem instanceof JsonArray) {
				for(JsonElement element : (JsonArray)elem) {
					if(element instanceof JsonObject) {
						JsonObject obj = (JsonObject)element;
						String name = obj.getAsJsonPrimitive("name").getAsString();
						String value = obj.getAsJsonPrimitive("value").getAsString();
						if(obj.has("signature"))
							properties.put(name, new Property(name, value, obj.getAsJsonPrimitive("signature").getAsString()));
						else
							properties.put(name, new Property(name, value));
					}
				}
			}

			return properties;
		}

		public JsonElement serialize(PropertyMap properties, Type type, JsonSerializationContext context) {
			JsonArray arr = new JsonArray();
			for(Property property : properties.values()) {
				JsonObject obj = new JsonObject();
				obj.addProperty("name", property.getName());
				obj.addProperty("value", property.getValue());
				if(property.getSignature() != null)
					obj.addProperty("signature", property.getSignature());
				arr.add(obj);
			}
			return arr;
		}
	}
}
