package proxy.util;

import java.lang.reflect.Type;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class VersionId {
	private final String name;
	private final int protocol;

	public VersionId(String name, int protocol) {
		this.name = name;
		this.protocol = protocol;
	}

	public String getName() {
		return this.name;
	}

	public int getProtocol() {
		return this.protocol;
	}

	public static class Serializer implements JsonDeserializer<VersionId>, JsonSerializer<VersionId> {
		public VersionId deserialize(JsonElement elem, Type type, JsonDeserializationContext context) throws JsonParseException {
			JsonObject obj = JsonUtils.getJsonObject(elem, "version");
			return new VersionId(JsonUtils.getString(obj, "name"), JsonUtils.getInt(obj, "protocol"));
		}

		public JsonElement serialize(VersionId version, Type type, JsonSerializationContext context) {
			JsonObject obj = new JsonObject();
			obj.addProperty("name", version.getName());
			obj.addProperty("protocol", (Number)Integer.valueOf(version.getProtocol()));
			return obj;
		}
	}
}